// micro:bit croc clip keeper - scruss, 2020-10

dimens = [52, 42, 1.6];
corner_r = 3;
hole_dia = 4;
holes =([
    [ 4.5  , 7 ],
    [ 4.5 + 10  , 7 ],
    [ 4.5 + 10 + 11.5  , 7 ],
    [ 4.5 + 10 + 11.5 + 11.5  , 7 ],
    [ 4.5 + 10 + 11.5 + 11.5 + 10  , 7 ]
]);

up=10; // how high the board can go without fouling components
$fn=64;
eps=1e-3;
layer = 0.2;
// appx total height
h_over = holes[0].y + (sqrt(3)/2) * hole_dia/2 + 2 * dimens.z + layer;

module outl(s) {
    // 2d outline of width s
    difference() {
        offset(r= s/2)children();
        offset(r=-s/2)children();
    }
}

module no_elephant(t=layer) {
    // I am very fond of Proboscidea,
    //  but I don't want their feet on my print bed
    // - scruss, 2020-10
    // assumes model is flat at z=0
    linear_extrude(height=t)outl(t)projection(cut=true)children();
}

module plain_board() {
    linear_extrude(height=dimens.z, center=true)difference()translate([corner_r, corner_r])offset(r=corner_r)square([dimens.x - 2*corner_r, dimens.y - 2*corner_r]);
}

module big_plain_board() {
    minkowski() {
        plain_board();
        sphere(d=layer, $fn=8);
    }
}

module board() {
    difference() {
        plain_board();
        for (i=[0:4]) {
            translate([holes[i].x, holes[i].y, 0])cylinder(d=hole_dia, h=dimens.z+eps, center=true);
        }
    }
}

module mini_ogive(r) {
    intersection() {
        circle(r=r);
        translate([r,0])circle(r=r);
        square(r);
    }
}

module aperture() {
    translate([-hole_dia/2,0])union() {
        translate([0,holes[0].y])mini_ogive(hole_dia);
        square([hole_dia, holes[0].y+eps]);
    }
}

module post_profile() {
    rotate_extrude()intersection() {
        offset(r=dimens.z)aperture();
        square(3 * holes[0].y);
    }
}

module outer() {
    hull() {
        post_profile();
        translate([dimens.x,0,0])post_profile();
    }
}

module outer_base() {
    // this is already expensive, so cut this early
    difference() {
        projection(cut=true)outer();
        offset(r=layer/2)translate([0, -dimens.z/2])square([dimens.x, dimens.z]);
    }
}

module nope() {
    no_elephant(layer)linear_extrude(height=eps)outer_base();
}

module cutout() {
    translate([0,0,h_over])mirror([0,0,1])scale([3/4,2,1])post_profile();
}

module keeper() {
    difference() {
	union() {
            linear_extrude(height=dimens.z)offset(r=dimens.z)hull()outer_base();
	    outer();
	}
        union() {
            translate([0,0,up])rotate([-90,0,0])big_plain_board();
            nope();
            for (i=[0:4]) {
                translate([holes[i].x,0,0])cutout();
            }
        }
    }
}

keeper();
